<?php

/**
 * @package     EasyStore.Site
 * @subpackage  EasyStore.Sample
 *
 * @copyright   Copyright (C) 2023 JoomShaper <https://www.joomshaper.com>. All rights reserved.
 * @license     GNU General Public License version 3; see LICENSE
 */

namespace JoomShaper\Plugin\EasyStore\Sample\Extension;


use Joomla\CMS\Log\Log;
use Joomla\Event\Event;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Application\CMSApplication;
use JoomShaper\Plugin\EasyStore\Sample\Utils\SampleConstants;
use JoomShaper\Component\EasyStore\Administrator\Plugin\PaymentGatewayPlugin;

// phpcs:disable PSR1.Files.SideEffects
\defined('_JEXEC') or die;
// phpcs:enable PSR1.Files.SideEffects

class SamplePayment extends PaymentGatewayPlugin
{
    /**
     * Verifies whether all the essential fields are configured before initiating the payment process, and  sets a boolean value,in the event arguments using the key `result`. 
     *
     * @return void -- Returns true if all the necessary field is not empty, otherwise, returns false.
     * 
     * @since 1.0.0
     */

    public function onBeforePayment(Event $event)
    {
        $constants              = new SampleConstants();
        $secretKey              = $constants->getSecretKey();
        $merchantKey            = $constants->getMerchantKey();
        $isRequiredFieldsFilled = !empty($secretKey) && !empty($merchantKey);
        
        $event->setArgument('result', $isRequiredFieldsFilled);
    }
    
    /**
     * Initiate an event that will lead to a redirection to the checkout page.
     *
     * @param Event $event -- The event object that contains cart data required for payment processing.
     * 
     * @since 1.0.0
     */

    public function onPayment(Event $event)
    {
        // Get the necessary data from `SampleConstants` which are needed to initiate payment process.
        $constants   = new SampleConstants();
        $secretKey   = $constants->getSecretKey();
        $merchantKey = $constants->getMerchantKey(); 
        $checkoutUrl = $constants->getUrl();
        
        /**
         * Extracting payment-related data from the event arguments.
         * If the subject is not set in the arguments, a default empty object is used.
         */
        $arguments   = $event->getArguments();
        $paymentData = $arguments['subject'] ?: new \stdClass();
        
        
        // Array representing the cart items.
        $products             = $paymentData->items; 
        // The overall transaction cost, including tax, shipping charge, and coupon discounts.  
        $totalPrice           = $paymentData->total_price; 
        // Easystore Order ID
        $orderID              = $paymentData->order_id;   
        // Name of the store which is set in the Store Settings. Default value is : `EasyStore`          
        $storeName            = $paymentData->store_name; 
        // Tax Amount for the order.         
        $tax                  = $paymentData->tax;  
        // Currency which is set in the Store Settings.         
        $currency             = $paymentData->currency;      
        // Country ISO alpha2/alpha3 value
        $country              = $paymentData->country; 
        // Shipping charge for the order.
        $shippingCharge       = $paymentData->shipping_charge;
        // Coupon Discount Amount for the order.
        $couponDiscountAmount = $paymentData->coupon_discount_amount;
        // Url of `EasyStore` Checkout Page
        $backToCheckoutPage   = $paymentData->back_to_checkout_page;
        // Decimal Separator which is set in the Store Settings. Default Value is '.'
        $decimalSeparator     = $paymentData->decimal_separator;
        // Thousand Separator which is set in the Store Settings. Default Value is ','
        $thousandsSeparator   = $paymentData->thousands_separator;    
        // User Name      
        $userName             = $paymentData->user_name;   
        // User Phone Number
        $userNumber           = $paymentData->user_number;
        // User Email Address
        $userEmail            = $paymentData->user_email; 


        /**
         * Define the URLs used for payment processing and notifications.
         */

        $redirectUrlAfterPayment = $constants->getSuccessUrl();  // User redirection after successfully completing the payment procedure on the payment portal.
        $cancelUrl               = $constants->getCancelUrl($orderID);  // User redirection upon payment cancellation
        $webhookUrl              = $constants->getWebHookUrl(); // URL for payment portal notifications

        try {
            
            /**
             *  In this try block, perform necessary tasks to create a checkout URL using the above variables for your payment, and then redirect the user. 
             * 
             *  📝 Ensure to transmit your $orderID as `metadata` or through any other supported process by your payment system while creating `$checkoutUrl`. This allows you to retrieve it in the onPaymentNotify() method for updating your payment status.
             */
            
            $this->app->redirect($checkoutUrl);
            
        } catch (\Throwable $error) {
            
            // Log the error message with the Log class.
            Log::add($error->getMessage(), Log::ERROR, 'sample.easystore');
            
            // Enqueue an error message to be displayed to the user.
            $this->app->enqueueMessage(Text::_('COM_EASYSTORE_SAMPLE_ERROR') . ' ' . $error->getMessage(), 'error');
            
            // Redirect the user to the checkout page to address the error.
            $this->app->redirect($paymentData->back_to_checkout_page);
        }
    }

    /**
     * Handles notifications received from the webhook / payment portal.
     *
     * @param Event $event -- The event object contains relevant data for payment notification, including Raw Payload, GET data, POST data, server variables, and a variable representing an instance with two methods.
     * @since 1.0.0
     */

    public function onPaymentNotify(Event $event)
    {      
        /**
         * Extracting payment-related notification data from the event arguments.
         * If the subject is not set in the arguments, a default empty object is used.
         */
        $arguments         = $event->getArguments();
        $paymentNotifyData = $arguments['subject'] ?: new \stdClass();
        
        // Raw payload from the webhook provider
        $rawPayload        = $paymentNotifyData->raw_payload;
        // Array of Get Parameters/Data from the webhook provider
        $getParameters     = $paymentNotifyData->get_data;
        // Array of Post Data from the webhook provider
        $postData          = $paymentNotifyData->post_data;
        // Array of Server Variables
        $serverVariables   = $paymentNotifyData->server_variables;
        // Represents an instance with two methods: `updateOrder` and `onOrderPlacementCompletion`.
        $order             = $paymentNotifyData->order;     
        
        $paymentStatus     = 'paid'; // The payment status will be received from the webhook provider.

        /**
         * Now, proceed with the necessary steps to verify the validity of the notification and ensure that it originated 
         * from your payment system and retrieve `order_id`, `payment_status` and `transactionID`.
         * 
         * Map the payment statuses to our system's supported statuses, which include "paid," "pending," "failed," and 
         * "canceled." Additionally, retrieve the error reasons for payments marked as failed or canceled.
        */
        
        try 
        {           
            $data = (object) [
                'id'                   => 1234, // Order ID
                'payment_status'       => 'paid', // Payment Status
                'payment_error_reason' => null,  // Add reason if payment status is failed or canceled
                'transaction_id'       => 'tran_5876349586795f', // Transaction ID
            ];
            
            // This method handles the tasks of updating the order and payment status, as well as sending emails based on the payment status. 
            $order->updateOrder($data);

            // If the payment status is 'paid', utilize the `onOrderPlacementCompletion` method. This method is responsible for reducing the quantity from the inventory and removing the cart items.           
            if ($paymentStatus === 'paid') {
                $order->onOrderPlacementCompletion();
            }
        
        } catch (\Throwable $error) {
            Log::add($error->getMessage(), Log::ERROR, 'sample.easystore');
        }
    }

}
